#
# $Id: mcSysInfo.pm,v 1.20 2002/03/15 21:39:35 mcooper Exp $
#
# Main mcSysInfo perl object
#

package mcSysInfo;
require 5.003;

use mcGlobals;
use mcMisc;
use mcDebug;
use mcGeneral;
use mcHardware;
use mcPartition;
use mcPrinter;
use mcSoftware;
use mcSysConf;
use mcKernel;

my @ClassDefault = ( 'general', 'device', 'software', 'sysconf',
		     'kernel', 'partition', 'printer' );

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"Program"} = undef;
    $self->{"InputFile"} = undef;
    $self->{"Class"} = [];
    $self->{"General"} = mcGeneral->New();
    $self->{"Hardware"} = mcHardware->New();
    $self->{"Software"} = mcSoftware->New();
    $self->{"Partition"} = mcPartition->New();
    $self->{"Printer"} = mcPrinter->New();
    $self->{"SysConf"} = mcSysConf->New();
    $self->{"Kernel"} = mcKernel->New();

    bless ($self, $class);

    return $self;
}

#
# Name/path of sysinfo program
#
sub Program {
    my $self = shift;

    if (!@_) {
	return $self->{"Program"};
    }

    $self->{"Program"} = shift;

    return;
}

#
# Input File
#
sub InputFile {
    my $self = shift;

    if (!@_) {
	return $self->{"InputFile"};
    }

    $self->{"InputFile"} = shift;

    return;
}

#
# Class info list
#
sub Class {
    my $self = shift;

    if (!@_) {
	my @a = @{ $self->{"Class"} };
	return ($#a < 0) ? @ClassDefault : @a;
    }

    @{ $self->{"Class"} } = @_;

    return;
}

sub General {
    my $self = shift;

    if (!@_) {
	return $self->{"General"};
    }

    $self->{"General"} = shift;

    return;
}

sub SysConf {
    my $self = shift;

    if (!@_) {
	return $self->{"SysConf"};
    }

    $self->{"SysConf"} = shift;

    return;
}

sub Kernel {
    my $self = shift;

    if (!@_) {
	return $self->{"Kernel"};
    }

    $self->{"Kernel"} = shift;

    return;
}

sub Hardware {
    my $self = shift;

    if (!@_) {
	return $self->{"Hardware"};
    }

    $self->{"Hardware"} = shift;

    return;
}

sub Software {
    my $self = shift;

    if (!@_) {
	return $self->{"Software"};
    }

    $self->{"Software"} = shift;

    return;
}

sub Partition {
    my $self = shift;

    if (!@_) {
	return $self->{"Partition"};
    }

    $self->{"Partition"} = shift;

    return;
}

sub Printer {
    my $self = shift;

    if (!@_) {
	return $self->{"Printer"};
    }

    $self->{"Printer"} = shift;

    return;
}

#
# Find SysInfo
#
sub FindSysInfo {
    my $self = shift;

    my $Program = $self->Program;
    if ($Program =~ /\//) {
	Debug("FindSysInfo: Using SysInfo->Program = <%s>", $Program);
	return $Program;
    }

    if (!length($Program)) {
	$Program = "sysinfo";
    }

    foreach my $Dir (split(':', $ENV{'PATH'})) {
	my $p = $Dir . "/" . $Program;
	Debug("FindSysInfo: Checking <%s>", $p);
	if (-x $p) {
	    return $p;
	}
    }

    Error("Cannot locate %s program in \$PATH.", $Program);
    return;
}

#
# Run sysinfo program and retrieve data
#
sub Retrieve {
    my $self = shift;
    my $Args = shift;
    my $ProgressFunc = $Args->{ProgressFunc};
    my $RedirectSTDERR = $Args->{RedirectSTDERR};

    Debug("Retrieving SysInfo data...");

    #
    # Build a hash table for speedy access
    #
    my @Classes = $self->Class;

    Debug("SysInfo->Retrieve: Classes=<%s>", join(', ', @Classes));
    my $ClassTable;
    foreach my $class (@Classes) {
	my $c = lc($class);
	$ClassTable->{$c} = $c;
    }

    my $SIFH = GetFH();
    my $SaveSTDERR;
    if (my $File = $self->InputFile) {
	Debug("Using inputfile <%s>", $File);
	if (!open($SIFH, "< $File")) {
	    Error("%s: Open for read failed: %s",  $File, $!);
	    return;
	}
    } else {
	my $Path = $self->FindSysInfo;
	if (!$Path) {
	    return;
	}

	my @Cmd;
	push(@Cmd, $Path);
	push(@Cmd, '-repsep'); 		push(@Cmd, "'|'");
	push(@Cmd, '-format'); 		push(@Cmd, "report");
	push(@Cmd, '-msglevel'); 	push(@Cmd, "all");
	if ($Debug) {
	    push(@Cmd, '-debug');
	}

	# Map 'hardware' -> 'device' for backwards compat
	my @cl = @Classes;
	for (my $i = 0; $i <= $#cl; ++$i) {
	    if (lc($cl[$i]) eq 'hardware') {
		$cl[$i] = 'device';
	    }

	}
	push(@Cmd, '-class');		push(@Cmd, lc(join(',', @cl)));

	my $Command = join(' ', @Cmd);
	Debug("Running <%s>", $Command);
	
	if ($RedirectSTDERR) {
	    $SaveSTDERR = GetFH();
	    if (!open($SaveSTDERR, ">&STDERR")) {
		Error("Cannot save STDERR: %s", $!);
		return;
	    }

	    my $fn = fileno($RedirectSTDERR);
	    if (!open(STDERR, "<&=$fn")) {
		Error("Redirect STDERR failed: %s", $!);
		return;
	    }
	}

	if (!open($SIFH, "$Command |")) {
	    Error("%s: Run command failed: %s", $Command, $!);
	    return;
	}
    }

    while (my $Line = <$SIFH>) {
	chop($Line);

	# Select seperator
	my $Sep = '\|';
	if ($Line =~ / \| /) {
	    $Sep = ' \| ';
	}
	if ($Line !~ /$Sep/) {
	    # Input lines must have at least 1 $Sep char
	    next;
	}

	my @Fields = split($Sep, $Line);
	my $Class = $Fields[0];

	# Special mapping
	if (Eq($Class, 'device')) {
	    $Class = 'hardware';
	}

	# Skip it if this class is not enabled
	if (!$ClassTable->{lc($Class)}) {
	    next;
	}

	if (Eq($Class, 'general')) {
	    $self->General->Decode(@Fields);
	} elsif (Eq($Class, 'hardware')) {
	    $self->Hardware->Decode(@Fields);
	} elsif (Eq($Class, 'software')) {
	    $self->Software->Decode(@Fields);
	} elsif (Eq($Class, 'partition')) {
	    $self->Partition->Decode(@Fields);
	} elsif (Eq($Class, 'printer')) {
	    $self->Printer->Decode(@Fields);
	} elsif (Eq($Class, 'sysconf')) {
	    $self->SysConf->Decode(@Fields);
	} elsif (Eq($Class, 'kernel')) {
	    $self->Kernel->Decode(@Fields);
	} else {
	    Debug("SysInfo->Retrieve: Class <%s> is undefined.", $Class);
	}
    }

    close($SIFH);

    if ($SaveSTDERR) {
	if (!open(STDERR, ">&$SaveSTDERR")) {
	    Error("Restore STDERR failed: %s", $!);
	}
    }

    Debug("Retrieve SysInfo data DONE");

    return $self;
}

1; # Make require/use happy
